/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Chimera code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Aaron Schulman <aschulm@umd.edu>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#import "RolloverImageButton.h"

@interface RolloverImageButton (Private)
  - (void)updateImage:(BOOL)inIsInside;
  - (BOOL)isMouseInside;
  - (void)removeTrackingRectFromView:(NSView *)inView;
@end

@implementation RolloverImageButton

- (id)initWithFrame:(NSRect)inFrame
{
  if ((self = [super initWithFrame:inFrame])) {
    mImage = nil;
    mHoverImage = nil;
    mTrackingTag = -1;
  }
  return self;
}

- (void)dealloc
{
  [mImage release];
  [mHoverImage release];
  [super dealloc];
}

- (void)removeFromSuperview
{
  [self removeTrackingRectFromView:[self superview]];
  [super removeFromSuperview];
}

- (void)setFrame:(NSRect)inFrame
{
  // save some time on code that attempts to set the frame to the same rectangle as the current frame
  if (!NSEqualRects(inFrame, [self frame])) {
    [super setFrame:inFrame];
    [self removeTrackingRectFromView:[self superview]];
          
    BOOL mouseInside = [self isMouseInside];
    mTrackingTag = [[self superview] addTrackingRect:[self frame] owner:self userData:nil assumeInside:mouseInside];
    [self updateImage:mouseInside];
  }
}

- (void)mouseEntered:(NSEvent*)theEvent
{
   // only act on the mouseEntered if the view is active or accepts the first mouse click
  if ([[self window] isKeyWindow] || [[self superview] acceptsFirstMouse:theEvent]) {
    if ([theEvent trackingNumber] == mTrackingTag) 
      [self updateImage:YES];
  }
}

- (void)mouseExited:(NSEvent*)theEvent
{
 // only act on the mouseExited if the view is active or accepts the first mouse click
  if ([[self window] isKeyWindow] || [[self superview] acceptsFirstMouse:theEvent]) {
    if ([theEvent trackingNumber] == mTrackingTag) 
      [self updateImage:NO];
  }
}

- (void)mouseDown:(NSEvent*)theEvent
{
  // if the mouse drags outside the frame with the left button down we want the image to appear as normal
  [self updateImage:NO];
  [super mouseDown:theEvent];
}

- (void)mouseUp:(NSEvent*)theEvent
{
  // if the mouse releases inside the button we want the hover image to appear
  [self updateImage:YES];
  [super mouseUp:theEvent];
}

- (void)setImage:(NSImage*)inImage
{
  if (mImage)
    [mImage release];
  mImage = [inImage retain];
  // set the image based on where the mouse currently is
  [self updateImage:[self isMouseInside]];
}

- (void)setHoverImage:(NSImage*)inImage
{
  if (mHoverImage)
    [mHoverImage release];
  mHoverImage = [inImage retain];
  // set the image based on where the mouse currently is
  [self updateImage:[self isMouseInside]];
}

@end

@implementation RolloverImageButton (Private)

- (void)updateImage:(BOOL)inIsInside
{
  if (inIsInside)
    [super setImage:mHoverImage];
  else
    [super setImage:mImage];
  // this is needed because part of the superview will appear malformed after button redraws unless it is set for update
  [[self superview] setNeedsDisplay:YES];
}

- (BOOL)isMouseInside
{
  NSPoint mousePointInWindow = [[self window] convertScreenToBase:[NSEvent mouseLocation]];
  NSPoint mousePointInView = [[self superview] convertPoint:mousePointInWindow fromView:nil];
  return NSMouseInRect(mousePointInView, [self frame], NO);
}

- (void)removeTrackingRectFromView:(NSView *)inView
{
  [inView removeTrackingRect:mTrackingTag];
  mTrackingTag = -1;
}

@end